# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2014 XCG Consulting (www.xcg-consulting.fr)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import osv, fields
import dateutil.rrule
import ast
import datetime
from weekdays.helpers import BusinessCalendar

# The dateutil.rrule.values are not stored because YEARLY is 0 and OpenERP
# thinks that the field is not set in that case.
_freq_values = [
    ('YEARLY', u"Yearly"),
    ('MONTHLY', u"Monthly"),
    ('WEEKLY', u"Weekly"),
    ('DAILY', u"Daily"),
    ('HOURLY', u"Hourly"),
    ('MINUTELY', u"Minutely"),
    ('SECONDLY', u"Secondly"),
]

# Dictionary that match stored values to dateutil.rrule values.
_freq_dict = {
    'YEARLY': dateutil.rrule.YEARLY,
    'MONTHLY': dateutil.rrule.MONTHLY,
    'WEEKLY': dateutil.rrule.WEEKLY,
    'DAILY': dateutil.rrule.DAILY,
    'HOURLY': dateutil.rrule.HOURLY,
    'MINUTELY': dateutil.rrule.MINUTELY,
    'SECONDLY': dateutil.rrule.SECONDLY,
}


class rrule_day(osv.Model):
    """this model is used by all implementers who need to define
    days that will be passed to our rrule engine.

    A good example of this would be a timesheet application that would
    need to define the start of the week and the day-off days per people.
    """

    _name = 'rrules.rrule_day'

    _columns = {
        'weekday': fields.integer('Weekday'),
        'name': fields.char('Name', translate=True),
    }


class rrule(osv.Model):
    _name = 'rrules.rrule'

    def get_rrule_args(self, cr, uid, ids, args, context=None):
        """
        :return dict: id/dict of args for the rrule of that id
        """
        lines = self.browse(cr, uid, ids, context)
        if args is None:
            args = {}
        res = {}
        for line in lines:
            rrule_args = {}
            rrule_args['freq'] = _freq_dict[line.freq]
            rrule_args['interval'] = line.interval
            if 'date_from' in args:
                rrule_args['dtstart'] = args['date_from']
            if 'date_end' in args:
                rrule_args['until'] = args['date_end']
            if line.bysetpos:
                rrule_args['bysetpos'] = ast.literal_eval(line.bysetpos)
            if line.bymonth:
                rrule_args['bymonth'] = ast.literal_eval(line.bymonth)
            if line.bymonthday:
                rrule_args['bymonthday'] = ast.literal_eval(line.bymonthday)
            if line.byyearday:
                rrule_args['byyearday'] = ast.literal_eval(line.byyearday)
            if line.byweekno:
                rrule_args['byweekno'] = ast.literal_eval(line.byweekno)
            if line.byweekday:
                rrule_args['byweekday'] = ast.literal_eval(line.byweekday)
            if line.byeaster:
                rrule_args['byeaster'] = ast.literal_eval(line.byeaster)
            res[line.id] = rrule_args
        return res

    def get_rrule(self, cr, uid, ids, args, context=None):
        """
        :return dict: id/dict of args for the rrule of that id
        """
        lines = self.get_rrule_args(cr, uid, ids, args, context)
        res = {}
        for line in lines:
            res[line] = dateutil.rrule.rrule(**lines[line])
        return res

    def get_rrule_today(
        self, cr, uid, ids, context=None, dayoff=None, dttoday=None
    ):
        # dttoday is an args for tests and preview
        if dttoday is None:
            dtoday = datetime.date.today()
            dttoday = datetime.datetime(dtoday.year, dtoday.month, dtoday.day)
        args = {
            'date_from': dttoday,
            'date_end': dttoday,
        }
        if dayoff:
            rrule_args = []
            for rrule_id in dayoff['holidays']:
                rrule_args.append(self.get_rrule_args(
                    cr, uid, [rrule_id.id], args, context)[rrule_id.id])
            # any week starting day is ok there
            business_calendar = BusinessCalendar(
                0, dayoff['weekoff'], rrule_args)
        res = {}
        lines = self.get_rrule_args(cr, uid, ids, args, context)
        for line in lines:
            rrule_args = lines[line]
            if dayoff:
                # evaluate dayoff for the period/freq including today
                if (
                    rrule_args['freq'] == dateutil.rrule.MONTHLY
                    or rrule_args['freq'] == dateutil.rrule.WEEKLY
                ):
                    # In the case of weekly, all the dates are not necessary
                    rrule_args['bymonthday'] = [
                        datetime_.day
                        for datetime_
                        in business_calendar.get_month_business_days(
                            dttoday.year, dttoday.month)
                    ]
                else:
                    raise NotImplementedError
            # get back rrules
            res[line] = dateutil.rrule.rrule(**rrule_args)
        return res

    _columns = {
        'name': fields.char(
            u"Name",
            size=64,
            translate=True,
            required=True,
        ),
        # From here, the field are in the same order as rrule doc
        'freq': fields.selection(
            _freq_values,
            string=u"Frequency",
            help=u"Frequency of the rrule.",
            required=True,
        ),
        # cache: no reason to let it be set by user
        'interval': fields.integer(
            string=u"Interval",
            help=u"The interval between each freq iteration. For example, when"
            " using YEARLY, an interval of 2 means once every two years, but"
            " with HOURLY, it means once every two hours. The default interval"
            " is 1.",
        ),
        # dtstart: not to be set by rrule, only when evaluating one
        # wkst is that useful? can be added if needed
        # count this should not be set, we’ll use dtstart/until
        # until: not set by rrule, only when evaluating one on a period
        'bysetpos': fields.char(
            u"By set pos",
            size=64,
            help=u"If given, it must be either an integer, or a sequence of"
            " integers, positive or negative. Each given integer will specify"
            " an occurrence number, corresponding to the nth occurrence of the"
            " rule inside the frequency period. For example, a bysetpos of -1"
            " if combined with a MONTHLY frequency, and a byweekday of (MO,"
            " TU, WE, TH, FR), will result in the last work day of every month"
            "."
        ),
        'bymonth': fields.char(
            u"By month",
            size=64,
            help=u"If given, it must be either an integer, or a sequence"
            " of integers, meaning the months to apply the recurrence to.",
        ),
        'bymonthday': fields.char(
            u"By month day",
            size=64,
            help=u"If given, it must be either an integer, or a sequence"
            " of integers, meaning the month days to apply the recurrence to.",
        ),
        'byyearday': fields.char(
            u"By year day",
            size=64,
            help=u"If given, it must be either an integer, or a sequence"
            " of integers, meaning the year days to apply the recurrence to.",
        ),
        'byweekno': fields.char(
            u"By week no",
            size=64,
            help=u"If given, it must be either an integer, or a sequence of"
            " integers, meaning the week numbers to apply the recurrence to."
            " Week numbers have the meaning described in ISO8601, that is,"
            " the first week of the year is that containing at least four days"
            " of the new year.",
        ),
        'byweekday': fields.char(
            u"By week day",
            size=64,
            help=u"If given, it must be an integer (0 == MO), a sequence of"
            " integers. When given, these variables will define the weekdays"
            " where the recurrence will be applied. It's also possible to use"
            " an argument n for the weekday instances, which will mean the nth"
            " occurrence of this weekday in the period. For example, with"
            " MONTHLY, or with YEARLY and BYMONTH, using FR(+1) in byweekday"
            " will specify the first Friday of the month where the recurrence"
            " happens. Notice that in the RFC documentation, this is specified"
            " as BYDAY, but was renamed to avoid the ambiguity of that keyword"
            ".",
        ),
        # TODO add byhour?
        # TODO add byminute?
        # TODO add bysecond?
        'byeaster': fields.char(
            u"By easter",
            size=64,
            help=u"If given, it must be either an integer, or a sequence of"
            " integers, positive or negative. Each integer will define an"
            " offset from the Easter Sunday. Passing the offset 0 to byeaster"
            " will yield the Easter Sunday itself. This is occidental Easter,"
            " not orthodox Easter",
        ),
        #'type': fields.many2many(
        #    'res.type',
        #),
    }

    _defaults = {
        'interval': 1,
    }
